/* This is the main window for our precision test.
 * It inherits from the QMainDialog class.
 * 
 * concept and design: Petr Felkel - felkepet@fel.cvut.cz
 *
 * programming: Petr Felkel   - felkepet@fel.cvut.cz
 *				Lukas Novosad - novosluk@fel.cvut.cz
 *
 * date: 21.10.2010
 *
 */

#include "exercise_3.h"


// constructor of the class
exercise_3::exercise_3(QWidget *parent, Qt::WFlags flags)
	: QMainWindow(parent, flags)
{
	ui.setupUi(this);
	drawArea.setParent(ui.drawAreaWidget);

	// create the labels for the status bar
	pointGeneratorLabel = new QLabel("Point distribution: ");
	pointGeneratorValueLabel = new QLabel();
	numberOfPointsLabel = new QLabel("Num. of points: ");
	numberOfPointsValueLabel = new QLabel();
	chAlgorithmLabel = new QLabel("CH algorithm: ");
	chAlgorithmValueLabel = new QLabel();
	timeMeasurementLabel = new QLabel("Time measured: ");
	timeMeasurementValueLabel = new QLabel();
	// add the labels to the status bar
	ui.statusBar->addPermanentWidget(pointGeneratorLabel);
	ui.statusBar->addPermanentWidget(pointGeneratorValueLabel,80);
	ui.statusBar->addPermanentWidget(numberOfPointsLabel);
	ui.statusBar->addPermanentWidget(numberOfPointsValueLabel,70);
	ui.statusBar->addPermanentWidget(chAlgorithmLabel);
	ui.statusBar->addPermanentWidget(chAlgorithmValueLabel,90);
	ui.statusBar->addPermanentWidget(timeMeasurementLabel);
	ui.statusBar->addPermanentWidget(timeMeasurementValueLabel,130);
	// remove the status bar grip from the lower right corner
	ui.statusBar->setSizeGripEnabled(false);
	
	// Qt connect statements
	QObject::connect(this,		SIGNAL(pointGeneratorChanged(PointGenerator)),	&drawArea,		SLOT(setPointGenerator(PointGenerator))				);
	QObject::connect(this,		SIGNAL(pointGeneratorChanged(PointGenerator)),	this,			SLOT(setPointGeneratorValueLabel(PointGenerator))	);
	QObject::connect(this,		SIGNAL(chAlgorithmChanged(ChAlgorithm)),	 	&drawArea,		SLOT(setChAlgorithm(ChAlgorithm))					);
	QObject::connect(this,		SIGNAL(chAlgorithmChanged(ChAlgorithm)),	 	this,			SLOT(setChAlgorithmValueLabel(ChAlgorithm))			);
	QObject::connect(this,		SIGNAL(numOfPointsChanged(long)),	 			&drawArea,		SLOT(setNumberOfPoints(long))						);
	QObject::connect(this,		SIGNAL(numOfPointsChanged(long)),				this,			SLOT(setNumberOfPointsValueLabel(long))					);
	QObject::connect(&drawArea, SIGNAL(timeMeasurementChanged(QString)),		this,			SLOT(setTimeMeasurementValueLabel(QString))			);

	// intialize the point generator, the number of points, and the CH algorithm
	setPointGeneratorOnDisc();
	setNumOfPoints100();
	setChAlg_AKL();
}


// default destructor of the class
exercise_3::~exercise_3()
{

}


/*
	This method shows the about dialog.
 */
void exercise_3::createAboutDialog()
{
	aboutDialog.setModal(true);
	aboutDialog.show();
}


// slots to select the point generator
void exercise_3::setPointGeneratorOnDisc()		{	ui.rb_On_Disc_3->setChecked(true); 		emit pointGeneratorChanged(IN_DISC);		}
void exercise_3::setPointGeneratorOnCircle()	{	ui.rb_On_Circle_3->setChecked(true); 	emit pointGeneratorChanged(ON_CIRCLE);		}
void exercise_3::setPointGeneratorInSquare()	{	ui.rb_In_Square_3->setChecked(true); 	emit pointGeneratorChanged(IN_SQUARE);		}
void exercise_3::setPointGeneratorOnSquare()	{	ui.rb_On_Square_3->setChecked(true); 	emit pointGeneratorChanged(ON_SQUARE);		}
void exercise_3::setPointGeneratorInRectangle()	{	ui.rb_In_Rectangle_3->setChecked(true);	emit pointGeneratorChanged(IN_RECTANGLE);	}
void exercise_3::setPointGeneratorOnSegment()	{	ui.rb_On_Segment_3->setChecked(true); 	emit pointGeneratorChanged(ON_SEGMENT);		}


// slots to select the CH algorithm
void exercise_3::setChAlg_AKL()				{	ui.rb_ch_Akl_4->setChecked(true);			emit chAlgorithmChanged( AKL );				}
void exercise_3::setChAlg_BYKAT()			{	ui.rb_ch_Bykat_4->setChecked(true);			emit chAlgorithmChanged( BYKAT );			}
void exercise_3::setChAlg_EDDY()			{	ui.rb_ch_Eddy_4->setChecked(true);			emit chAlgorithmChanged( EDDY );			}
void exercise_3::setChAlg_GRAHAM_ANDREW()	{	ui.rb_ch_GrahamAndrew_4->setChecked(true);	emit chAlgorithmChanged( GRAHAM_ANDREW );	}
void exercise_3::setChAlg_JARVIS()			{	ui.rb_ch_Jarvis_4->setChecked(true);		emit chAlgorithmChanged( JARVIS );			}
void exercise_3::setChAlg_MY_GRAHAM()		{	ui.rb_ch_My_Graham_4->setChecked(true); 	emit chAlgorithmChanged( MY_GRAHAM );		}
void exercise_3::setChAlg_MY_JARVIS()		{	ui.rb_ch_My_Jarvis_4->setChecked(true);		emit chAlgorithmChanged( MY_JARVIS );		}
void exercise_3::setChAlg_MY_CHAN()			{	ui.rb_ch_My_Chan_5->setChecked(true);		emit chAlgorithmChanged( MY_CHAN );		}


// slots to select the number of points
void exercise_3::setNumOfPoints10()			{	ui.rb_10->setChecked(true);			emit numOfPointsChanged( 10);		}
void exercise_3::setNumOfPoints100()		{	ui.rb_100->setChecked(true);		emit numOfPointsChanged( 100);		}
void exercise_3::setNumOfPoints1000()		{	ui.rb_1000->setChecked(true);		emit numOfPointsChanged( 1000);		}
void exercise_3::setNumOfPoints10000()		{	ui.rb_10000->setChecked(true);		emit numOfPointsChanged( 10000);	}
void exercise_3::setNumOfPoints100000()		{	ui.rb_100000->setChecked(true);		emit numOfPointsChanged( 100000);	}
void exercise_3::setNumOfPoints1000000()	{	ui.rb_1000000->setChecked(true);	emit numOfPointsChanged( 1000000);	}
void exercise_3::setNumOfPoints10000000()	{	ui.rb_10000000->setChecked(true);	emit numOfPointsChanged( 10000000);	}


/*
	This slot updates the status bar label for the currently selected number of points.
 */
void exercise_3::setNumberOfPointsValueLabel(long numberOfPoints)
{
	// update the number of points value label
	numberOfPointsValueLabel->setText(QString("%1").arg(numberOfPoints));
}


/*
	This slot updates the status bar label for the currently selected point generator.
 */
void exercise_3::setPointGeneratorValueLabel(PointGenerator pointGenerator)
{
	QString text;

	switch( pointGenerator )
	{
	case IN_DISC:
		text = QString("in a disc");
		break;
	case ON_CIRCLE:
		text = QString("on a circle");
		break;
	case IN_SQUARE:
		text = QString("in a square");
		break;
	case ON_SQUARE:
		text = QString("on a square");
		break;
	case IN_RECTANGLE:
		text = QString("in a rectangle");
		break;
	case ON_SEGMENT:
		text = QString("on a segment");
		break;
	default:
		break;
	}

	// update the point generator value label
	pointGeneratorValueLabel->setText(text);
}


/*
	This slot updates the status bar label for the currently selected convex hull algorithm.
 */
void exercise_3::setChAlgorithmValueLabel(ChAlgorithm chAlgorithm)
{
	QString text;

	switch( chAlgorithm ) 
	{
	case AKL:
			text = QString("Akl");
			break;
	case BYKAT:
			text = QString("Bykat");
			break;
	case EDDY:
			text = QString("Eddy");
			break;
	case GRAHAM_ANDREW:
			text = QString("Graham_andrew");
			break;
	case JARVIS:
			text = QString("Jarvis");
			break;
	case MY_GRAHAM:
			text = QString("My_Graham");
			break;
	case MY_JARVIS:
			text = QString("My_Jarvis");
			break;
	case MY_CHAN:
			text = QString("My_Chan");
			break;
	default:
		break;
	}

	// update the CH algorithm value label
	chAlgorithmValueLabel->setText(text);
}


/*
	This slot updates the status bar label for the current time measurment.
 */
void exercise_3::setTimeMeasurementValueLabel(QString timeMeasurement)
{
	// update the time measuremnt value label
	timeMeasurementValueLabel->setText(timeMeasurement);
}